/**
 * Copyright (c) 2017-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.admin.junit;

import java.net.URL;

import org.ow2.petals.admin.api.artifact.Artifact;
import org.ow2.petals.admin.api.artifact.ArtifactException;
import org.ow2.petals.admin.api.artifact.ArtifactState;
import org.ow2.petals.admin.api.artifact.ArtifactState.State;
import org.ow2.petals.admin.api.artifact.ArtifactUtils;
import org.ow2.petals.admin.api.artifact.lifecycle.ArtifactLifecycle;
import org.ow2.petals.admin.api.exception.ArtifactAdministrationException;
import org.ow2.petals.admin.api.exception.ArtifactDeployedException;
import org.ow2.petals.admin.api.exception.ArtifactNotDeployedException;
import org.ow2.petals.admin.api.exception.ArtifactNotFoundException;
import org.ow2.petals.admin.api.exception.ArtifactStartedException;
import org.ow2.petals.admin.api.exception.ArtifactStoppedException;
import org.ow2.petals.admin.topology.Container;

public abstract class AbstractArtifactStateLifecycleMock<A extends Artifact & ArtifactState> implements ArtifactLifecycle {

    /**
     * Container on which the Petals Administration API is connected
     */
    protected final Container container;

    protected A artifact;

    public AbstractArtifactStateLifecycleMock(final A artifact, final Container container) {
        this.artifact = artifact;
        this.container = container;
    }

    protected abstract A findArtefactInContainer();

    protected abstract void registerArtefactInContainerForDeploy(A artifact) throws ArtifactAdministrationException;

    protected abstract void removeArtefactInContainerForUndeploy(A artifact) throws ArtifactAdministrationException;

    protected void setState(State targetState) throws ArtifactNotFoundException, ArtifactNotDeployedException,
            ArtifactStartedException, ArtifactStoppedException, ArtifactDeployedException {
        A find = findArtefactInContainer();
        if (find != null) {
            State currentState = find.getState();

            // for start()
            if (targetState == State.STARTED) {
                if (currentState == State.STARTED) {
                    throw new ArtifactStartedException(find.getType(), find.getName());
                }
                if (currentState == State.LOADED) {
                    throw new ArtifactNotDeployedException(find.getType(), find.getName());
                }
            }

            // for stop()
            if (targetState == State.STOPPED) {
                if (currentState == State.LOADED) {
                    throw new ArtifactNotDeployedException(find.getType(), find.getName());
                }
                if (currentState == State.SHUTDOWN || currentState == State.STOPPED) {
                    throw new ArtifactStoppedException(artifact.getType(), artifact.getName());
                }
            }

            // for components.uninstall()
            if (targetState == State.LOADED) {
                if (currentState == State.STARTED) {
                    throw new ArtifactStartedException(find.getType(), find.getName());
                }
                if (currentState == State.LOADED) {
                    throw new ArtifactNotDeployedException(find.getType(), find.getName());
                }
            }

            // for components.install()
            if (targetState == State.SHUTDOWN) {
                if (currentState == State.SHUTDOWN || currentState == State.STOPPED || currentState == State.STARTED) {
                    throw new ArtifactDeployedException(find.getName(), currentState.toString());
                }
            }

            find.setState(targetState);
        } else {
            throw new ArtifactNotFoundException(artifact.getType(), artifact.getName(), artifact.getVersion());
        }
    }

    @Override
    public void updateState() throws ArtifactNotFoundException {
        A found = findArtefactInContainer();
        if (found != null) {
            artifact.setState(found.getState());
        } else {
            throw new ArtifactNotFoundException(artifact.getType(), artifact.getName(), artifact.getVersion());
        }
    }

    @Override
    public void deploy(final URL artifactUrl) throws ArtifactAdministrationException {
        A found = findArtefactInContainer();

        if (found != null) {
            // in shutdown or loaded state, it wil be undeployed first
            // TODO why shutdown?!
            if (found.getState() == State.SHUTDOWN || found.getState() == State.LOADED) {
                removeArtefactInContainerForUndeploy(found);
            } else {
                throw new ArtifactDeployedException(found.getName(), found.getState().toString());
            }
        }

        try {
            artifact = (A) ArtifactUtils.createArtifact(artifactUrl);
        } catch (ArtifactException e) {
            throw new ArtifactAdministrationException("Can't create an artifact from URL " + artifactUrl, e);
        }

        artifact.setState(State.SHUTDOWN);

        registerArtefactInContainerForDeploy(artifact);
    }

    @Override
    public void start() throws ArtifactAdministrationException {
        try {
            setState(State.STARTED);
        } catch (ArtifactStoppedException | ArtifactDeployedException e) {
            throw new AssertionError("impossible", e);
        }
    }

    @Override
    public void stop() throws ArtifactAdministrationException {
        try {
            setState(State.STOPPED);
        } catch (ArtifactDeployedException | ArtifactStartedException e) {
            throw new AssertionError("impossible", e);

        }
    }

    @Override
    public void undeploy() throws ArtifactAdministrationException {
        A found = findArtefactInContainer();
        if (found != null) {
            if (found.getState() == State.STARTED) {
                throw new ArtifactStartedException(found.getType(), found.getName());
            }
            removeArtefactInContainerForUndeploy(found);
        } else {
            throw new ArtifactNotFoundException(artifact.getType(), artifact.getName(), artifact.getVersion());
        }
    }

}
