/**
 * Copyright (c) 2016-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.admin.junit;

import java.util.Properties;

import org.ow2.petals.admin.api.ArtifactAdministration;
import org.ow2.petals.admin.api.ContainerAdministration;
import org.ow2.petals.admin.api.EndpointDirectoryAdministration;
import org.ow2.petals.admin.api.PetalsAdministration;
import org.ow2.petals.admin.api.artifact.lifecycle.ArtifactLifecycleFactory;
import org.ow2.petals.admin.api.conf.ArtifactUrlRewriter;
import org.ow2.petals.admin.api.exception.ConnectionFailedException;
import org.ow2.petals.admin.api.exception.ContainerAdministrationException;
import org.ow2.petals.admin.api.exception.NoConnectionException;
import org.ow2.petals.admin.junit.exception.NoDomainRegisteredException;
import org.ow2.petals.admin.topology.Container;
import org.ow2.petals.admin.topology.Container.PortType;

public class PetalsAdministrationMock implements PetalsAdministration {

    /**
     * Name of an host generating an error '{@link ContainerAdministrationException}' when connecting.
     */
    public static final String UNKNOWN_HOST = "192.168.0.250";

    private final PetalsAdministrationApi api;

    /**
     * Container on which the Petals Administration API is connected
     */
    private Container currentContainer = null;

    /**
     * Flag to simulate an established connection
     */
    private volatile boolean isConnected = false;

    public PetalsAdministrationMock(PetalsAdministrationApi api) {
        this.api = api;
    }

    @Override
    public void connect(final String host, final int port, final String user, final String password)
            throws ContainerAdministrationException {
        if (UNKNOWN_HOST.equals(host)) {
            throw new ContainerAdministrationException("Unknown Host");
        } else if (api.isNullDomain()) {
            isConnected = true;
        } else if (api.getDomain() == null) {
            throw new ContainerAdministrationException(new NoDomainRegisteredException());
        } else {
            for (final Container container : api.getDomain().getContainers()) {
                if (container.getState().equals(Container.State.UNREACHABLE)) {
                    throw new ConnectionFailedException("Unable to connect to " + container.getHost());
                }
                if (container.getHost().equals(host) && container.getPorts().get(PortType.JMX).intValue() == port) {
                    if (container.getJmxUsername().equals(user) && container.getJmxPassword().equals(password)) {
                        isConnected = true;
                        this.currentContainer = container;
                        return;
                    } else {
                        throw new ContainerAdministrationException("Invalid Credentials");
                    }
                }
            }
            // No container matching host:port was found in the domain
            isConnected = false;
            throw new ContainerAdministrationException("Unknown Host");
        }
    }

    @Override
    public boolean isConnected() throws ContainerAdministrationException {
        return isConnected;
    }

    @Override
    public void disconnect() throws NoConnectionException, ContainerAdministrationException {
        isConnected = false;
        this.currentContainer = null;
    }

    @Override
    public ArtifactAdministration newArtifactAdministration() {
        return new ArtifactAdministrationMock(api.getDomain(), this, ArtifactUrlRewriter.NOOP);
    }

    @Override
    public ArtifactAdministration newArtifactAdministration(ArtifactUrlRewriter artifactUrlRewriter) {
        return newArtifactAdministration();
    }

    @Override
    public ContainerAdministration newContainerAdministration() {
        return new ContainerAdministrationMock(this);
    }

    @Override
    public EndpointDirectoryAdministration newEndpointDirectoryAdministration() {
        final EndpointDirectoryAdministrationMock endpointDirectoryAdministrationMock = new EndpointDirectoryAdministrationMock();
        if (api.getEndpoints() != null) {
            endpointDirectoryAdministrationMock.setEndpoints(api.getEndpoints());
        }
        return endpointDirectoryAdministrationMock;
    }

    @Override
    public ArtifactLifecycleFactory newArtifactLifecycleFactory(ArtifactUrlRewriter artifactUrlRewriter) {
        return new ArtifactLifecycleFactoryMock(this.currentContainer);
    }

    @Override
    public ArtifactLifecycleFactory newArtifactLifecycleFactory() {
        return new ArtifactLifecycleFactoryMock(this.currentContainer);
    }

    PetalsAdministrationApi getApi() {
        return api;
    }

    public Properties getLocalConfiguration() {
        return currentContainer == null ? null : currentContainer.getLocalConfiguration();
    }
}
